//
//  QuizViewController+widget.swift
//  BJVideoPlayerUI
//
//  Created by ney on 2022/11/2.
//  Copyright © 2022 BaijiaYun. All rights reserved.
//

import BJVideoPlayerCore

fileprivate let onePixel = 1.0 / UIScreen.main.scale

internal extension QuizViewController {
    // MARK: - widget
    class QuizAnswerResultView: UIView {
        var result = false
        var modelData = BJVQuizItem()
        func updateResult(_ result: Bool, data: BJVQuizItem) {
            self.result = result
            self.modelData = data
            resultImageView.image = UIImage.bjpu_imageNamed(result ? "player_quiz_result_right" : "player_quiz_result_wrong")
            resultDescriptionLabel.text = result ? data.rightHint : data.wrongHint
            videoBackHintLabel.text = data.backInterval <= 0 ? "" : BJLLocalizedString("视频将回退\(NSInteger(data.backInterval))秒前")
            videoBackHintLabel.isHidden = result || data.backInterval <= 0
        }
        
        func didShown() -> Bool {
            return self.superview != nil
        }
        
        func answerIDs() -> String {
            var ids = [String]()
            for i in self.modelData.options where i.isCorrect {
                ids.append(String(i.displayOrder))
            }
            return String(ids.joined(separator: ","))
        }
        
        required init() {
            super.init(frame: .zero)
            setupUI()
        }
        
        required init?(coder: NSCoder) {
            fatalError("init(coder:) has not been implemented")
        }
        
        private let resultImageView = UIImageView { view in
        }
        private let resultDescriptionLabel = UILabel { lb in
            lb.font = .systemFont(ofSize: 14)
            lb.textColor = .bjl_color(withHexString: "#4A4A4A")
            lb.numberOfLines = 0
        }
        private let videoBackHintLabel = UILabel { lb in
            lb.font = .systemFont(ofSize: 12)
            lb.textColor = .bjl_color(withHexString: "#FF1F49")
        }
        
        private func setupUI() {
            resultImageView.layout(in: self) { make in
                make.centerX.equalToSuperview()
                make.top.equalTo(16)
            }
            videoBackHintLabel.layout(in: self) { make in
                make.centerX.equalTo(resultImageView)
                make.bottom.equalTo(-20)
            }
            resultDescriptionLabel.layout(in: self) { make in
                make.centerX.equalTo(resultImageView)
                make.top.equalTo(resultImageView.snp.bottom).offset(6)
                make.left.greaterThanOrEqualTo(40)
                make.right.lessThanOrEqualTo(-40)
                make.bottom.equalTo(videoBackHintLabel.snp.top).offset(-8)
            }
        }
    }
    
    class QuizView: UIView {
        required init() {
            super.init(frame: .zero)
            setupUI()
        }
        
        required init?(coder: NSCoder) {
            fatalError("init(coder:) has not been implemented")
        }
        
        private let quizImageView = UIImageView { view in
        }
        
        private let quizImageViewHeight = UIDevice.isPhone ? 0.26 * min(UIScreen.main.bounds.size.height, UIScreen.main.bounds.size.width) : 112.0
        private let quizTitleLabel = UILabel { lb in
            lb.numberOfLines = 0
            lb.font = .systemFont(ofSize: 14)
            lb.textColor = .bjl_color(hex: 0x03081A)
        }
        let quizOptionsView = OptionsView { view in
        }
        
        func setupUI() {
            quizImageView.layout(in: self) { make in
                make.top.equalTo(16)
                make.left.equalTo(20)
                
                make.height.equalTo(quizImageViewHeight)
                make.width.equalTo(quizImageView.snp.height).multipliedBy(200.0 / quizImageViewHeight)
            }
            quizTitleLabel.layout(in: self) { make in
                make.left.equalTo(20)
                make.top.equalTo(20)
                make.right.lessThanOrEqualToSuperview().offset(-20)
            }
            quizOptionsView.remakeLayout(in: self) { make in
                make.top.equalTo(quizTitleLabel.snp.bottom).offset(8)
                make.left.right.equalToSuperview()
                make.bottom.equalTo(0)
                make.height.equalTo(130)
            }
        }
        
        func updateOptionsUI(quizData: BJVQuizItem) {
            let hasImage = quizData.imageURL.count > 0
            quizImageView.image = nil
            if hasImage {
                quizTitleLabel.snp.remakeConstraints { make in
                    make.top.equalTo(12 + quizImageViewHeight + 12)
                    make.left.equalTo(20)
                    make.right.lessThanOrEqualToSuperview().offset(-20)
                }
                // setup image
                quizImageView.bjl_setImage(with: URL(string: quizData.imageURL), placeholder: nil, completion: nil)
            }
            else {
                quizTitleLabel.snp.remakeConstraints { make in
                    make.top.equalTo(20)
                    make.left.equalTo(20)
                    make.right.lessThanOrEqualToSuperview().offset(-20)
                }
            }
            quizImageView.isHidden = !hasImage
            
            quizTitleLabel.text = BJLLocalizedString(quizData.singleChoice ? "【单选题】" : "【多选题】") + " " + quizData.title
            quizOptionsView.reloadOptions(quizData.options, isSingleChoice: quizData.singleChoice)
        }
    }
    
    class TitleView: UIView {
        private let titleLabel = UILabel { lb in
            lb.text = BJLLocalizedString("答题")
            lb.font = .systemFont(ofSize: 14)
            lb.textColor = .bjl_color(hex: 0x03081A)
        }
        private let bottomLine = UIView { view in
            view.backgroundColor = .bjl_color(hex: 0x9FA8B5)
        }
        
        override init(frame: CGRect) {
            super.init(frame: frame)
            
            self.backgroundColor = .bjl_color(hex: 0xB0BEC5).withAlphaComponent(0.1)
            
            titleLabel.layout(in: self) { make in
                make.centerY.equalToSuperview()
                make.left.equalTo(20)
            }
            bottomLine.layout(in: self) { make in
                make.left.bottom.right.equalToSuperview()
                make.height.equalTo(onePixel)
            }
        }
        
        required init?(coder: NSCoder) {
            fatalError("init(coder:) has not been implemented")
        }
    }
    
    class BottomView: UIView {
        
        static let commitButtonTitle = BJLLocalizedString("提交")
        static let finishButtonTitle = BJLLocalizedString("完成")
        
        var leftButtonEventHandler: ((BottomView, UIButton) -> Void)?
        var rightButtonEventHandler: ((BottomView, UIButton) -> Void)?
        
        private let topLine = UIView { view in
            view.backgroundColor = .bjl_color(hex: 0x9FA8B5)
        }
        private let leftButton = UIButton { btn in
            btn.layer.cornerRadius = 6.0
            btn.layer.masksToBounds = true
            btn.titleLabel?.font = UIFont.systemFont(ofSize: 12.0)
            btn.contentHorizontalAlignment = .center
            btn.backgroundColor = .bjl_color(withHexString: "#B0BEC5", alpha: 0.3)
            btn.setTitleColor(.bjl_color(withHexString: "#333333"), for: .normal)
            btn.setTitle(BJLLocalizedString("跳过"), for: .normal)
        }
        private let rightButton = UIButton { btn in
            btn.layer.cornerRadius = 6.0
            btn.layer.masksToBounds = true
            btn.titleLabel?.font = UIFont.systemFont(ofSize: 12.0)
            btn.contentHorizontalAlignment = .center
            btn.backgroundColor = .bjl_color(withHexString: "#1795FF")
            btn.setTitleColor(.white, for: .normal)
            btn.setTitle(commitButtonTitle, for: .normal)
        }
        
        func showLeftButton(_ show: Bool) {
            leftButton.isHidden = !show
        }
        
        func updateRightButtonTitle(_ title: String) {
            rightButton.setTitle(BJLLocalizedString(title), for: .normal)
        }
        
        override init(frame: CGRect) {
            super.init(frame: frame)
            
            self.backgroundColor = .bjl_color(hex: 0xB0BEC5).withAlphaComponent(0.1)
            
            topLine.layout(in: self) { make in
                make.left.top.right.equalToSuperview()
                make.height.equalTo(onePixel)
            }
            rightButton.layout(in: self) { make in
                let size = UIDevice.isPad ? CGSize(width: 120, height: 36) : CGSize(width: 96, height: 28)
                make.size.equalTo(size)
                make.centerY.equalToSuperview()
                make.right.equalTo(-16)
            }
            leftButton.layout(in: self) { make in
                make.size.equalTo(rightButton)
                make.centerY.equalToSuperview()
                make.right.equalTo(rightButton.snp.left).offset(-12)
            }
            leftButton.bjl_addHandler { [weak self] btn in
                guard let self = self else { return }
                self.leftButtonEventHandler?(self, btn)
            }
            rightButton.bjl_addHandler { [weak self] btn in
                guard let self = self else { return }
                self.rightButtonEventHandler?(self, btn)
            }
        }
        
        required init?(coder: NSCoder) {
            fatalError("init(coder:) has not been implemented")
        }
    }
    
    class OptionsView: UIView, UITableViewDataSource, UITableViewDelegate {
        func reloadOptions(_ options: [BJVOptionsItem], isSingleChoice: Bool) {
            self.dataSource = options
            self.isSingleChoice = isSingleChoice
            unselectAll()
            tableView.reloadData()
        }
        
        private var dataSource = [BJVOptionsItem]()
        private var isSingleChoice = false
        
        private let tableView: UITableView = {
            let view = UITableView(frame: .zero, style: .plain)
            view.separatorStyle = .none
            view.backgroundColor = .white
            view.register(OptionsViewCell.self, forCellReuseIdentifier: "OptionsViewCell")
            return view
        }()
        
        override init(frame: CGRect) {
            super.init(frame: frame)
            
            tableView.delegate = self
            tableView.dataSource = self
            
            tableView.layout(in: self) { make in
                make.edges.equalToSuperview()
            }
        }
        
        func answerIDs() -> String {
            var ids = [String]()
            for i in dataSource where i.selected {
                ids.append(String(i.displayOrder))
            }
            return String(ids.joined(separator: ","))
        }
        
        func hasNotSelect() -> Bool {
            for i in dataSource where i.selected { return false }
            return true
        }
        
        required init?(coder: NSCoder) {
            fatalError("init(coder:) has not been implemented")
        }
        
        func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
            return dataSource.count
        }
        
        func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
            let cell = tableView.dequeueReusableCell(withIdentifier: "OptionsViewCell", for: indexPath)
            guard let optionCell = cell as? OptionsViewCell else { return UITableViewCell() }
            let data = dataSource[indexPath.row]
            optionCell.update(title: data.title, index: indexPath.row)
            optionCell.updateSelectionState(data.selected)
            optionCell.selectionStyle = .none
            return optionCell
        }
        
        func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
            guard let cell = tableView.cellForRow(at: indexPath), let optionCell = cell as? OptionsViewCell else { return }
            let data = dataSource[indexPath.row]
            if isSingleChoice {
                unselectAll()
                data.selected = true
                tableView.reloadData()
            }
            else {
                data.selected = !data.selected
                optionCell.updateSelectionState(data.selected)
            }
        }
        
        func unselectAll() {
            for i in dataSource { i.selected = false }
        }
    }
    
    class OptionsViewCell: UITableViewCell {
        private let indexLabel = UILabel { lb in
            lb.backgroundColor = .bjl_color(withHexString: "#1795FF")
            lb.textColor = .white
            lb.textAlignment = .center
            lb.font = .systemFont(ofSize: 14, weight: .medium)
            lb.layer.masksToBounds = true
            lb.layer.cornerRadius = 10
        }
        private let titleLabel = UILabel { lb in
            lb.backgroundColor = .clear
            lb.textColor = .bjl_color(withHexString: "#333333")
            lb.font = .systemFont(ofSize: 14, weight: .regular)
            lb.numberOfLines = 0
        }
        private let selectionButton = UIButton { btn in
            btn.isUserInteractionEnabled = false
            btn.setImage(UIImage.bjpu_imageNamed("player_quiz_options_right"), for: .normal)
            btn.snp.makeConstraints { make in
                make.width.height.equalTo(20)
            }
            btn.isHidden = true
        }
        
        override init(style: UITableViewCell.CellStyle, reuseIdentifier: String?) {
            super.init(style: style, reuseIdentifier: reuseIdentifier)
            
            indexLabel.layout(in: self.contentView) { make in
                make.left.equalTo(20)
                make.centerY.equalToSuperview()
                make.width.height.equalTo(20)
            }
            selectionButton.layout(in: self.contentView) { make in
                make.centerY.equalToSuperview()
                make.right.equalTo(-10)
                make.top.greaterThanOrEqualToSuperview()
                make.bottom.lessThanOrEqualToSuperview()
            }
            titleLabel.layout(in: self.contentView) { make in
                make.left.equalTo(indexLabel.snp.right).offset(10)
                make.right.lessThanOrEqualTo(selectionButton.snp.left).offset(-10)
                make.top.equalTo(8)
                make.bottom.equalTo(-8)
            }
        }
        
        required init?(coder: NSCoder) {
            fatalError("init(coder:) has not been implemented")
        }
        
        func update(title: String, index: Int) {
            titleLabel.text = title
            let alphabetTable = Array((Unicode.Scalar("A").value...Unicode.Scalar("Z").value)).map {
                String(Unicode.Scalar($0) ?? Unicode.Scalar(0))
            }
            if index >= 0 && index < alphabetTable.count {
                indexLabel.text = alphabetTable[index]
            }
            else {
                indexLabel.text = "-"
            }
        }
        
        func updateSelectionState(_ on: Bool) {
            selectionButton.isHidden = !on
            contentView.backgroundColor = on ? .bjl_color(withHexString: "#f5f6f7") : .white
        }
    }
}
