//
//  BJPUVideoOptions.h
//  BJVideoPlayerUI
//
//  Created by HuangJie on 2018/4/24.
//  Copyright © 2018年 BaijiaYun. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <BJVideoPlayerCore/BJVPlayerManager.h>
#import <BJLiveBase/BJLYYModel.h>
#import <BJPlayerUIBase/BJPUStopPlayModel.h>
#import <BJVideoPlayerCore/BJVCustomWaterMarkModel.h>

@interface BJPUVideoOptions: NSObject <BJLYYModel>

@property (nonatomic, assign) BJVPlayerType playerType; // 播放器类型
@property (nonatomic, assign) BOOL videoFit; // 视频的缩放类型
@property (nonatomic, assign) BOOL advertisementEnabled; // 播放广告，暂不支持
@property (nonatomic, assign) BOOL autoplay; // 自动播放
@property (nonatomic, assign) BOOL sliderDragEnabled; // 能否拖动进度条
@property (nonatomic, assign) BOOL enableVideoRate; // 能否调整倍速
@property (nonatomic, assign) BOOL playTimeRecordEnabled; // 开启记忆播放
@property (nonatomic, assign) BOOL encryptEnabled; // 开启加密
@property (nonatomic, assign) BOOL backgroundAudioEnabled; // 开启后台播放，必须在工程的 background modes 中添加 audio 才会生效
@property (nonatomic, assign) BOOL remoteControlEnabled; // 开启远程控制
@property (nonatomic, strong) UIImage *remoteControlImage; // 远程控制图片
@property (nonatomic, assign) NSTimeInterval initialPlayTime; // 起始播放时间
@property (nonatomic, assign) NSInteger maxWatchTime; // 点播观看最大可拖拽时间，单位秒，到达时间点之前可以拖动，之后不可以任意拖动，取值大于0有效
@property (nonatomic, assign) BOOL isLandscapeLayout; // 默认布局是否是横屏，默认为 YES
@property (nonatomic, assign) BOOL autoPlayNextAlbumVideo; // 点播专辑是否自动播放下一个视频，默认YES
@property (nonatomic, assign) BOOL externalAlnumShow; // 外部控制是否展示合集按钮

/**
 开启画中画
 note: 当前的播放器类型为 BJVPlayerType_AVPlayer 且 backgroundAudioEnabled == NO 时, 设置 pictureInPictureEnabled 才有效
 note: 当设置 backgroundAudioEnabled 为 YES 时, 内部会把 pictureInPictureEnabled 置为 NO
 #discussion 默认为YES
 */
@property (nonatomic, assign) BOOL pictureInPictureEnabled;

/**
 偏好清晰度列表
 
 #discussion 优先使用此列表中的清晰度播放在线视频，优先级按数组元素顺序递减
 #discussion 列表元素为清晰度的标识字符串，现有标识符：low（标清），high（高清），superHD（超清），720p，1080p，audio（纯音频），可根据实际情况动态扩展
 #discussion 此设置对播放本地视频无效
 */
@property (nonatomic, strong) NSArray<NSString *> *preferredDefinitionList;

/**
 能否拖动进度条到未观看过的位置
 
 #discussion 开启之后，拖动进度条会弹框提示
 #discussion 可自定义设置弹框的文字提示标题，最大 16 字符
 #discussion 可自定义设置弹框的文字提示内容，最大 48 字符
 #discussion 可自定义设置弹框的icon 和按钮文本，按钮文本最大 20 字符

 */
@property (nonatomic, assign) BOOL sliderAdvanceEnabled;
@property (nonatomic, strong) BJPUStopPlayModel *sliderDragStopPlayModel;

/**
 试听时间，单位秒
 
 #discussion 开启之后，拖动进度条会弹框提示
 #discussion 可自定义设置弹框的文字提示标题，最大 16 字符
 #discussion 可自定义设置弹框的文字提示内容，最大 48 字符
 #discussion 可自定义设置弹框的icon 和按钮文本，按钮文本最大 20 字符
 */
@property (nonatomic, assign) NSInteger auditionTime;
@property (nonatomic, strong) BJPUStopPlayModel *auditionTimeStopPlayModel;

/**
 自定义水印
 #discussion 自定义优先级高于原配置项
 */
@property (nonatomic, strong) BJVCustomWaterMarkModel *waterMarkModel;

/**
 第三方用户名和编号
 
 #discussion 用于上报统计
 */
@property (nonatomic, strong) NSString *userName;
@property (nonatomic, strong) NSString *userNumber;

@end
