//
//  BJPUViewController+media.m
//  BJVideoPlayerUI
//
//  Created by HuangJie on 2018/3/9.
//

#import "BJPUViewController+media.h"
#import "BJPUViewController+protected.h"

@implementation BJPUViewController (media)

#pragma mark - callbacks

- (void)setMediaCallbacks {
    bjl_weakify(self);
    // 播放
    [self.mediaControlView setMediaPlayCallback:^{
        bjl_strongify(self);
        [self play];
    }];

    // 上一个
    [self.mediaControlView setPreMediaPlayCallback:^{
        bjl_strongify(self);
        self.currentAlbumIndex = self.currentAlbumIndex - 1;
        if (self.currentAlbumIndex < 0) {
            self.currentAlbumIndex = 0;
            return;
        }
        [self playWithVideoInfo:self.videoAlbumList[self.currentAlbumIndex]];
    }];

    // 下一个
    [self.mediaControlView setNextMediaPlayCallback:^{
        bjl_strongify(self);
        self.currentAlbumIndex = self.currentAlbumIndex + 1;
        if (self.currentAlbumIndex > [self.videoAlbumList count]) {
            self.currentAlbumIndex = 0;
        }
        [self playWithVideoInfo:self.videoAlbumList[self.currentAlbumIndex]];
    }];

    [self.mediaControlView setShowAlbumListCallback:^(BOOL show){
        bjl_strongify(self);
        if (self.videoOptions.externalAlnumShow) {
            if (self.externalAlbumClickCallback) {
                self.externalAlbumClickCallback(show);
            }
            return;
        }
        
        if (!self.VideoListViewController) {
            self.VideoListViewController = [[BJPVideoListViewController alloc] initWithVid:self.videoID list:self.videoAlbumList];
            self.VideoListViewController.closeCallback = ^() {
                bjl_strongify(self);
                self.videoAlbumListLayer.hidden = YES;
            };
            self.VideoListViewController.clickCallback = ^(BJVPlayinfoItem * model, NSInteger index) {
                bjl_strongify(self);
                if ([model.videoID isEqualToString:self.videoID]) {
                    return;
                }
                
                for (NSInteger index = 0; index < [self.videoAlbumList count]; index ++) {
                    BJVPlayinfoItem *item = self.videoAlbumList[index];
                    if ([item.videoID isEqualToString:model.videoID]) {
                        self.currentAlbumIndex = index;
                        break;
                    }
                }
                [self playWithVideoInfo:model];
                self.videoAlbumListLayer.hidden = YES;
            };
            [self bjl_addChildViewController:self.VideoListViewController superview:self.videoAlbumListLayer];
        }
        BOOL isHorizontal = BJPUIsHorizontalUI(self);

        if (isHorizontal) {
            [self.VideoListViewController.view bjl_remakeConstraints:^(BJLConstraintMaker *_Nonnull make) {
                make.top.right.bottom.equalTo(self.videoAlbumListLayer);
                make.width.equalTo(@(300.0));
            }];
//            [self.mediaControlView setControlsHidden:YES];
            
            [self.gradientLayer removeFromSuperlayer];
            self.gradientLayer.frame = self.videoAlbumListLayer.frame;
            [self.videoAlbumListLayer.layer insertSublayer:self.gradientLayer atIndex:0];
            
            self.videoAlbumListLayer.hidden = !self.videoAlbumListLayer.hidden;
        }
        else {
            self.videoAlbumListLayer.hidden = YES;
        }
    }];
    
    // 暂停
    [self.mediaControlView setMediaPauseCallback:^{
        bjl_strongify(self);
        [self pause];
    }];

    // seek
    [self.mediaControlView setMediaSeekCallback:^(NSTimeInterval toTime) {
        bjl_strongify(self);
        NSTimeInterval time = toTime;
        if (self.videoOptions.maxWatchTime > 0 && toTime > self.videoOptions.maxWatchTime) {
            time = self.videoOptions.maxWatchTime;
        }
        if (!self.videoOptions.sliderAdvanceEnabled && toTime > self.currentMaxWatchTime) {
            [self showSliderDragStopPlayView];
            return;
        }
        
        [self seekToTime:time];
    }];
    // 切换倍速
    [self.mediaControlView setShowRateListCallback:^ {
        bjl_strongify(self);
        BOOL show = !self.mediaSettingView.hidden;
        if (show && self.currentOptions == self.rateList) {
            self.mediaSettingView.hidden = YES;
        }
        else {
            [self showRateList];
        }
    }];

    // 切换清晰度
    [self.mediaControlView setShowDefinitionListCallback:^ {
        bjl_strongify(self);
        BOOL show = !self.mediaSettingView.hidden;
        if (show && self.currentOptions == self.playerManager.playInfo.definitionList) {
            self.mediaSettingView.hidden = YES;
        }
        else {
            [self showDefinitionList];
        }
    }];

    // 字幕
    [self.mediaControlView setShowSubtitleListCallback:^{
        bjl_strongify(self);
        [self showSubtitleView];
    }];

    // 缩放
    [self.mediaControlView setScaleCallback:^(BOOL horizon) {
        bjl_strongify(self);
        if (self.fullScreenButtonCallback) {
            self.fullScreenButtonCallback(horizon);
        }
        [self updateConstriantsWithLayoutType:horizon ? BJVPlayerViewLayoutType_Horizon : BJVPlayerViewLayoutType_Vertical];
    }];

    // 选择 倍速／清晰度
    [self.mediaSettingView setSelectCallback:^(NSUInteger selectIndex) {
        bjl_strongify(self);
        if (self.currentOptions == self.rateList) {
            id option = [self optionOrNilAtIndex:selectIndex];
            if (option && [option respondsToSelector:@selector(floatValue)]) {
                CGFloat rate = [option floatValue] ?: 1.0;
                if (self.videoOptions.maxWatchTime > 0 && self.playerManager.currentTime >= self.videoOptions.maxWatchTime) {
                    rate = 1;
                }
                [self.playerManager setRate:rate];
            }
        }
        else if (self.currentOptions == self.playerManager.playInfo.definitionList) {
            [self.playerManager changeDefinitionWithIndex:selectIndex];
        }
        [self hideView:self.mediaSettingView withDuration:1];
    }];

    // 刷新重试
    [self.reloadView setReloadCallback:^{
        bjl_strongify(self);
        if (self.isLocalVideo) {
            [self playWithDownloadItem:self.downloadItem];
        }
        else {
            [self playWithVid:self.videoID token:self.token];
        }
    }];
}

#pragma mark - action

- (void)play {
    [self.playerManager play];
}

- (void)pause {
    [self.playerManager pause];
}

- (void)seekToTime:(NSTimeInterval)time {
    time = MIN(time, self.playerManager.duration - 0.01);
    [self.quizVC playerShouldSeekToTimeOffset:time];
    [self.playerManager seek:time];
}

- (void)showRateList {
    self.currentOptions = self.rateList;
    CGFloat height = [self.currentOptions count] * [self.mediaSettingView perHeight];
    CGFloat videoContentContainerHeight = self.videoContentContainer.bounds.size.height;
    if (height + 80 > videoContentContainerHeight && videoContentContainerHeight > 0) {
        height = videoContentContainerHeight - 80;
    }
    [self.mediaSettingView bjl_remakeConstraints:^(BJLConstraintMaker *make) {
        make.bottom.equalTo(self.mediaControlView.bjl_top).offset(-2);
        make.width.equalTo(@72.0);
        make.height.equalTo(@(height));
        make.centerX.equalTo(self.mediaControlView.rateButton.bjl_centerX);
    }];

    [self.mediaSettingView updateWithSettingOptons:[self getRateOptions]
                                       selectIndex:self.rateIndex];
    [self showMediaSettingView];
}

- (void)showDefinitionList {
    self.currentOptions = self.playerManager.playInfo.definitionList;
    CGFloat height = [self.currentOptions count] * [self.mediaSettingView perHeight];
    CGFloat videoContentContainerHeight = self.videoContentContainer.bounds.size.height;
    if (height + 80 > videoContentContainerHeight && videoContentContainerHeight > 0) {
        height = videoContentContainerHeight - 80;
    }

    [self.mediaSettingView bjl_remakeConstraints:^(BJLConstraintMaker *make) {
        make.bottom.equalTo(self.mediaControlView.bjl_top).offset(-2);
        make.width.equalTo(@72.0);
        make.height.equalTo(@([self.currentOptions count] * [self.mediaSettingView perHeight]));
        make.centerX.equalTo(self.mediaControlView.definitionButton.bjl_centerX);
    }];
    [self.mediaSettingView updateWithSettingOptons:[self getDefinitionOptions]
                                       selectIndex:self.definitionIndex];
    [self showMediaSettingView];
}

- (void)showMediaSettingView {
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.5 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        self.mediaSettingView.hidden = NO;
    });
}

#pragma mark - media setting

- (NSArray *)getRateOptions {
    self.rateIndex = 0;
    NSMutableArray *rateOptions = [[NSMutableArray alloc] init];
    for (int i = 0; i < self.rateList.count; i++) {
        CGFloat rate = [[self.rateList objectAtIndex:i] floatValue];
        NSString *option = [NSString stringWithFormat:@"%.2f", rate];
        if (fabs(rate - self.playerManager.rate) < 0.1) {
            self.rateIndex = i;
        }
        [rateOptions addObject:option];
    }
    return [NSArray arrayWithArray:rateOptions];
}

- (NSArray *)getDefinitionOptions {
    NSArray *definitionList = self.playerManager.playInfo.definitionList;
    BJVDefinitionInfo *currdefinitionInfo = self.playerManager.currDefinitionInfo;
    NSMutableArray *definitionOptions = [[NSMutableArray alloc] init];
    for (int i = 0; i < definitionList.count; i++) {
        BJVDefinitionInfo *definitionInfo = [definitionList objectAtIndex:i];
        if ([definitionInfo.definitionKey isEqualToString:currdefinitionInfo.definitionKey]) {
            self.definitionIndex = i;
        }
        [definitionOptions addObject:definitionInfo.definitionName ?: @""];
    }
    return definitionOptions;
}

#pragma mark - private

- (id)optionOrNilAtIndex:(NSInteger)index {
    if (index >= 0 && index < self.currentOptions.count) {
        return [self.currentOptions objectAtIndex:index];
    }
    return nil;
}

@end
