//
//  BJPUViewController.h
//  BJVideoPlayerUI
//
//  Created by DLM on 2017/4/26.
//
//

#import <UIKit/UIKit.h>
#import "BJPUVideoOptions.h"
#import "BJPUAppearance.h"
#import "BJPUSignInModel.h"
#import <BJVideoPlayerCore/BJVLamp.h>

typedef NS_ENUM(NSUInteger, BJVPlayerViewLayoutType) {
    BJVPlayerViewLayoutType_Vertical,
    BJVPlayerViewLayoutType_Horizon
};

NS_ASSUME_NONNULL_BEGIN

@interface BJPUViewController: UIViewController

@property (nonatomic, assign) BJVPlayerViewLayoutType layoutType;
@property (nonatomic, readonly) BJVPlayerManager *playerManager;
@property (nonatomic, copy) void (^_Nullable cancelCallback)(void);
@property (nonatomic, copy) void (^_Nullable screenLockCallback)(BOOL locked);
@property (nonatomic, copy) void (^_Nullable fullScreenButtonCallback)(BOOL isFullScreen);
@property (nonatomic, copy) void (^_Nullable auditionAlertCallback)(UIView *alertView, BOOL refuse);
@property (nonatomic, copy) void (^_Nullable externalAlbumClickCallback)(BOOL show);

// 系统调用- supportedInterfaceOrientations 时，会返回此值。未设置时为0，会调用父类的实现
@property (nonatomic, assign) UIInterfaceOrientationMask supportedOrientations;

// 默认显示 导航栏返回 按钮
@property (nonatomic, assign) BOOL showNavigationBackButton;

// 是否在全屏/非全屏时，自动调整设备方向，默认为 YES
@property (nonatomic, assign) BOOL autoAdjustDeviceOrientationByFullScreenState;

- (instancetype)initWithVideoOptions:(BJPUVideoOptions *)videoOptions;

/** 更新配置 **/
- (void)updateVideoOptions:(BJPUVideoOptions *)videoOptions;

/** 在线视频播放 */
- (void)playWithVid:(NSString *)vid token:(NSString *)token;

/** 本地视频播放 */
- (void)playWithDownloadItem:(BJVDownloadItem *)downloadItem;

/** 在线视频播放点播专辑 */
- (void)playWithVideoAlbumNumber:(NSString *)number ;

/**
 设置跑马灯
 
 如果官网后台的 云点播-播放器设置-跑马灯设置 没有开启显示跑马灯 或者 customLamp.content == nil, 则无法显示跑马灯, customLamp为更高优先级
 */
@property (nonatomic, nullable) BJVLamp *customLamp;

/**
 用于点播弹题的 user number，用于唯一标志用户
 */
@property (nonatomic, copy, nullable) NSString *quizUserNumber;

/**
 用于点播弹题的 user name
 */
@property (nonatomic, copy, nullable) NSString *quizUserName;

/**
 用于自定义签到的数据
 注意：这里的自定义签到数据对长度有限制：
 * 标题：最大 16 字符
 * 副标题：最大 48 字符
 * 按钮：最大 20 字符
 */
@property (nonatomic, copy) NSArray <BJPUSignInModel *> *signInList;

/**
 用户点击签到弹框的回调
 */
@property (nonatomic, copy) void (^_Nullable userDidSignInCallback)(BJPUViewController *vc, BJPUSignInModel *signInData);

/**
 分享按钮的点击回调，如果此回调不设置，则不会显示分享按钮
 */
@property (nonatomic, copy) void (^_Nullable shareButtonTappedCallback)(BJPUViewController *vc);
@end

NS_ASSUME_NONNULL_END
