//
//  PUAppDelegate.m
//  BJPlayerManagerUI
//
//  Created by oushizishu on 06/02/2017.
//  Copyright (c) 2017 oushizishu. All rights reserved.
//

#import <BJLiveBase/BJL_EXTScope.h>
#import <BJLiveBase/NSObject+BJL_M9Dev.h>
#import <BJLiveBase/UIKit+BJLHandler.h>
#import <BJLiveBase/UIAlertController+BJLAddAction.h>

#import <BJVideoPlayerCore/BJVPlayerManager.h>
#import <BJVideoPlayerCore/BJVideoPlayerCore.h>
#import <BJVideoPlayerCore/BJVRoom.h>
#import <BJVideoPlayerCore/BJVDownloadManager.h>
#import <BJVideoPlayerCore/BJVTokenManager.h>

#import "PUAppDelegate.h"
#import "PUMainViewController.h"
#import "PUPlayOptionsManager.h"

#import "UIViewController+PUUtil.h"
#import "UIWindow+PUMotion.h"

#import "BJVDownloadViewController.h"

@interface PUAppDelegate () <BJVRequestTokenDelegate, BJLDownloadManagerClassDelegate>

@property (nonatomic, assign) UIBackgroundTaskIdentifier bgTask;

@end

@implementation PUAppDelegate

- (instancetype)init {
    self = [super init];
    if (self) {
        self.allowRotation = YES;
    }
    return self;
}

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    [PUMainViewController loadDeployType];
    [[BJVAppConfig sharedInstance] setPrivateDomainPrefix:[PUPlayOptionsManager getPrivatePrefix]];
    [BJVTokenManager setTokenDelegate:self];
    [BJVDownloadManager setClassDelegate:self];
    
    self.window = [[UIWindow alloc] initWithFrame:[UIScreen mainScreen].bounds];
    self.window.backgroundColor = [UIColor whiteColor];

    UINavigationController *navi = [[UINavigationController alloc] initWithRootViewController:[PUMainViewController new]];
    self.window.rootViewController = navi;
    [self.window makeKeyAndVisible];
    return YES;
}

- (void)applicationDidBecomeActive:(UIApplication *)application {
    // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.x
}

- (void)applicationWillTerminate:(UIApplication *)application {
    // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
}

- (UIInterfaceOrientationMask)application:(UIApplication *)application supportedInterfaceOrientationsForWindow:(nullable UIWindow *)window {
    if (!self.allowRotation) {
        return UIInterfaceOrientationMaskLandscape;
    }
    return UIInterfaceOrientationMaskAll;
}

#pragma mark - <BJLDownloadManagerClassDelegate>

- (void)downloadManager:(BJVDownloadManager *)downloadManager
URLSessionConfiguration:(NSURLSessionConfiguration *)configuration {
#if DEBUG
    // test change configuration, #see `refreshURLSessionWithCompletion:` in `BJVDownloadViewController.m`
    static BOOL allowsCellularAccess = YES;
    configuration.allowsCellularAccess = allowsCellularAccess; // The default value is YES.
    allowsCellularAccess = !allowsCellularAccess;
#endif
    // configuration.HTTPMaximumConnectionsPerHost = 4; // The default value is 6 in macOS, or 4 in iOS.
}

#pragma mark - <BJVRequestTokenDelegate>

- (void)requestTokenWithVideoID:(NSString *)videoID
                     completion:(void (^)(NSString * _Nullable token, NSError * _Nullable error))completion {
    NSString *key = videoID ?: @"";
    
    completion(BJLDownloadVideoTokens()[key], nil);
    
    // [self requestTokenWithKey:key completion:completion];
}

- (void)requestTokenWithClassID:(NSString *)classID
                      sessionID:(nullable NSString *)sessionID
                     completion:(void (^)(NSString * _Nullable token, NSError * _Nullable error))completion {
    NSString *separator = sessionID.length ? @"-" : @"";
    NSString *key = [NSString stringWithFormat:@"%@%@%@", classID ?: @"", separator, sessionID ?: @""];
    
    completion(BJLDownloadPlaybackTokens()[key], nil);
    
    // [self requestTokenWithKey:key completion:completion];
}

- (void)requestTokenWithKey:(NSString *)key completion:(void (^)(NSString * _Nullable token, NSError * _Nullable error))completion {
    NSDictionary *videos = BJLDownloadVideoTokens();
    NSDictionary *playbacks = BJLDownloadPlaybackTokens();
    NSString *token = videos[key] ?: playbacks[key];
    
    UIAlertController *alert = [UIAlertController
                                alertControllerWithTitle:[NSString stringWithFormat:@"requtst token for %@", key]
                                message:token
                                preferredStyle:UIAlertControllerStyleAlert];
    
    [alert bjl_addActionWithTitle:@"success" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        completion(token, nil);
    }];
    
    [alert bjl_addActionWithTitle:@"failure" style:UIAlertActionStyleCancel handler:^(UIAlertAction * _Nonnull action) {
        completion(nil, [NSError errorWithDomain:NSURLErrorDomain code:NSURLErrorCancelled userInfo:nil]);
    }];
    
    [self.window.rootViewController presentViewController:alert animated:YES completion:nil];
}

@end
